#include "config.h"
#include "parsing.h"
#include <arpa/inet.h>
#include <netinet/ip_icmp.h>
#include <signal.h>
#include <netinet/in.h>
#include <stdbool.h>
#include <stdint.h>
#include <stdio.h>
#include <string.h>
#include <sys/socket.h>
#include <unistd.h>

bool loop = true;

static void signal_handler(int code)
{
	(void)code;
	loop = false;
}

int init_socket(const struct config *conf)
{
    int sock_fd = socket(AF_INET, SOCK_RAW, IPPROTO_ICMP);
    if (sock_fd == -1)
    {
        dprintf(2, "ft_ping: open socket failed maybe missing root\n");
        return -1;
    }

    struct timeval timeout;      
    timeout.tv_sec = conf->timeout;
    timeout.tv_usec = 0;
    
    if (setsockopt (sock_fd, SOL_SOCKET, SO_RCVTIMEO, &timeout,
                sizeof timeout) < 0)
    {
        close(sock_fd);
        dprintf(2, "ft_ping: set socket options failed\n");
        return -1;
    }

    if (setsockopt (sock_fd, SOL_SOCKET, SO_SNDTIMEO, &timeout,
                sizeof timeout) < 0)
    {
        close(sock_fd);
        dprintf(2, "ft_ping: set socket options failed\n");
        return -1;
    }

    return sock_fd;
}

uint16_t checksum(const uint16_t *data, int len)
{
	uint32_t sum = 0;
	int i = 0;

	while (len > 1)
	{
		sum += data[i++];
		len -= 2;
	}
	if (len == 1)
		sum += ((uint8_t *)data)[i];

	while (sum >> 16)
		sum = (sum & 0xFFFF) + (sum >> 16);
	return (~sum);
}

int send_packet(int sock_fd, const char* host, uint64_t seq, const struct config *conf)
{
    (void) conf;
    struct sockaddr_in addr;
    inet_aton(host, &addr.sin_addr);
    
    struct icmphdr icmp_hdr;
    memset(&icmp_hdr, 0, sizeof(icmp_hdr));
    icmp_hdr.type = ICMP_ECHO;
    icmp_hdr.un.echo.id = getpid();
    icmp_hdr.un.echo.sequence = seq;
    icmp_hdr.checksum = checksum((uint16_t*)&icmp_hdr, sizeof(icmp_hdr));

    if (sendto(sock_fd, &icmp_hdr, sizeof(icmp_hdr), 0, (struct sockaddr*)&addr, sizeof(addr)) == -1)
    {
        dprintf(2, "ft_printf: failed to send packet\n");
        return 1;
    }
    return 0;
}

int recv_packet(int sock_fd, const char *host, uint64_t seq, const struct config *conf)
{
    (void) conf;
    (void) host;
    char buf[128];
    struct sockaddr_in addr; 

    socklen_t len = sizeof(addr);

    int n = recvfrom(sock_fd, buf, sizeof(buf), 0,
                     (struct sockaddr *)&addr, &len);
    if (n == -1)
        return 1;
    (void) seq;
    return 0;
}

int main(int ac, char **av)
{
    (void)ac;

    signal(SIGINT, signal_handler);

    struct config conf;
    set_default_config(&conf);
    const char *host = parsing((const char * const *) av, &conf);
    if (host == NULL)
        return 1;

    const int sock_fd = init_socket(&conf);
    if (sock_fd == -1)
        return 2;

    static uint64_t seq = 1;
    while (loop)
    {
        if (send_packet(sock_fd, host, seq, &conf))
        {
            close(sock_fd);
            return 1;
        };
        if (recv_packet(sock_fd, host, seq, &conf))
        {
            close(sock_fd);
            return 1;
        }
        seq++;
        sleep(conf.interval);
    }

    printf("PING TA %s\n", host);

    close(sock_fd);

    return 0;
}