#include "config.h"

#include <stdio.h>
#include <ctype.h>
#include <string.h>
#include <stdint.h>
#include <stdbool.h>

static int parse_int (const char *arg, uint64_t* ptr)
{
    *ptr = 0;
    for (uint64_t i = 0; arg[i] != '\0'; i++)
    {
        char c = arg[i];
        if (!isdigit(c))
        {
            dprintf(2, "ft_ping: %s: is not a valid number\n", arg);
            return 1;
        }
        *ptr += *ptr * 10 + c - '0';
    }
    return 0;
}

static int parse_count(const char *arg, struct config *conf)
{
    if (arg == NULL)
    {
        dprintf(2, "ft_ping: -c must be follow by the number of count\n");
        return 1;
    }
    return parse_int(arg, &conf->count);
}

static int parse_interval(const char *arg, struct config *conf)
{
    if (arg == NULL)
    {
        dprintf(2, "ft_ping: -i must be follow by the number of interval\n");
        return 1;
    }
    return parse_int(arg, &conf->interval);
}

const char *parsing(const char * const *args, struct config *conf)
{
    const char *arg;
    const char *host = NULL;

    for (uint64_t i = 1; args[i] != NULL; i++)
    {
        arg = args[i];
        if (strcmp(arg, "-v") == 0)
            conf->verbose = true;
        else if (strcmp(arg, "-q") == 0)
            conf->quiet = true;
        else if (strcmp(arg, "-c") == 0)
        {
            if (parse_count(args[++i], conf))
                return NULL;
        }
        else if (strcmp(arg, "-i") == 0)
        {
            if (parse_interval(args[++i], conf))
                return NULL;
        }
        else {
            host = arg;
        }
    }
    if (host == NULL)
        dprintf(2, "ft_ping: destination address is missing\n");
    return host;
}